@@ -94,6 +94,14 @@ par_setenv(name, value)
 	const char *name;
 	register const char *value;
 {
+#ifdef WIN32
+	char* p = (char*)malloc((size_t)(strlen(name) + strlen(value) + 2));
+	if (!p)
+		return (-1);
+	sprintf(p, "%s=%s", name, value);
+	_putenv(p);
+	return (0);
+#else
 	extern char **environ;
 	static int alloced = 0;         /* if allocated space before */
 	register char *c;
@@ -137,6 +145,7 @@ par_setenv(name, value)
 	for (c = environ[offset]; (*c = *name++) && *c != '='; ++c);
 	for (*c++ = '='; (*c++ = *value++););
 	return (0);
+#endif
 }
 
 /*
@@ -147,6 +156,9 @@ static void
 par_unsetenv(name)
 	const char *name;
 {
+#ifdef WIN32
+	par_setenv(name, "");
+#else
 	extern char **environ;
 	register char **p;
 	int offset;
@@ -155,4 +167,5 @@ par_unsetenv(name)
 		for (p = &environ[offset];; ++p)
 			if (!(*p = *(p + 1)))
 				break;
+#endif
 }
