@@ -143,7 +143,7 @@ This is just a zip file beginning with the magic string "C<PK\003\004>".
 
 =item * Ending section
 
-The pre-computed cache name.  A pack('Z40') string of the value of -T 
+The pre-computed cache name.  A pack('Z40') string of the value of -T
 (--tempcache) or the hash of the file, followed by C<\0CACHE>.  The hash
 of the file is calculated with L<Digest::SHA>.
 
@@ -154,13 +154,44 @@ followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
 
 =cut
 
-my ($par_temp, $progname, @tmpfile);
+my ($PAR_MAGIC, $FILE_offset_size, $cache_name_size);
+# NOTE: must initialize them in BEGIN as they are used in BEGIN below
+BEGIN {
+    $PAR_MAGIC = "\nPAR.pm\n";
+    $FILE_offset_size = 4;   # pack("N")
+    $cache_name_size = 40;
+    $PKZIP_MAGIC = "PK\003\004";
+}
+
+
+# Search $fh for the "\nPAR.pm\n signature backward from the end of the file
+sub find_par_magic
+{
+    my ($fh) = @_;
+
+    my $chunk_size = 64 * 1024;
+    my $buf;
+    my $size = -s $fh;
+
+    my $pos = ($size-1) - ($size-1) % $chunk_size;      # NOTE: $pos is a multiple of $chunk_size
+    while ($pos >= 0) {
+        seek $fh, $pos, 0;
+        read $fh, $buf, $chunk_size + length($PAR_MAGIC);
+        if ((my $i = rindex($buf, $PAR_MAGIC)) >= 0) {
+            return $pos + $i;
+        }
+        $pos -= $chunk_size;
+    }
+    return -1;
+}
+
+my ($par_temp, $progname, @tmpfile, %ModuleCache);
 END { if ($ENV{PAR_CLEAN}) {
     require File::Temp;
     require File::Basename;
     require File::Spec;
     my $topdir = File::Basename::dirname($par_temp);
-    outs(qq{Removing files in "$par_temp"});
+    outs(qq[Removing files in "$par_temp"]);
     File::Find::finddepth(sub { ( -d ) ? rmdir : unlink }, $par_temp);
     rmdir $par_temp;
     # Don't remove topdir because this causes a race with other apps
@@ -180,8 +211,10 @@ END { if ($ENV{PAR_CLEAN}) {
             SUFFIX => '.cmd',
             UNLINK => 0,
         );
+        my $filename = $tmp->filename;
 
-        print $tmp "#!/bin/sh
+        print $tmp <<"...";
+#!/bin/sh
 x=1; while [ \$x -lt 10 ]; do
    rm -rf '$par_temp'
    if [ \! -d '$par_temp' ]; then
@@ -190,17 +223,18 @@ x=1; while [ \$x -lt 10 ]; do
    sleep 1
    x=`expr \$x + 1`
 done
-rm '" . $tmp->filename . "'
-";
-            chmod 0700,$tmp->filename;
-        my $cmd = $tmp->filename . ' >/dev/null 2>&1 &';
+rm '$filename'
+...
         close $tmp;
+
+        chmod 0700, $filename;
+        my $cmd = "$filename >/dev/null 2>&1 &";
         system($cmd);
-        outs(qq(Spawned background process to perform cleanup: )
-             . $tmp->filename);
+        outs(qq[Spawned background process to perform cleanup: $filename]);
     }
 } }
 
+
 BEGIN {
     Internals::PAR::BOOT() if defined &Internals::PAR::BOOT;
 
@@ -211,60 +245,66 @@ _par_init_env();
 my $quiet = !$ENV{PAR_DEBUG};
 
 # fix $progname if invoked from PATH
-my %Config = (
+my %sys = (
     path_sep    => ($^O =~ /^MSWin/ ? ';' : ':'),
     _exe        => ($^O =~ /^(?:MSWin|OS2|cygwin)/ ? '.exe' : ''),
     _delim      => ($^O =~ /^MSWin|OS2/ ? '\\' : '/'),
 );
 
 _set_progname();
+outs(qq[\$progname = "$progname"]);
+
 _set_par_temp();
+outs(qq[\$ENV{PAR_TEMP} = "$ENV{PAR_TEMP}"]);
 
 # Magic string checking and extracting bundled modules {{{
-my ($start_pos, $data_pos);
-{
+my ($start_pos, $start_of_FILE_section);
+MAGIC: {
     local $SIG{__WARN__} = sub {};
 
     # Check file type, get start of data section {{{
-    open _FH, '<', $progname or last;
-    binmode(_FH);
+    unless (open _FH, '<:raw', $progname) {
+        outs(qq[Can't read from file "$progname"]);  # don't use $! here as it requires Errno.pm
+        last MAGIC;
+    }
 
     # Search for the "\nPAR.pm\n signature backward from the end of the file
-    my $buf;
-    my $size = -s $progname;
-    my $offset = 512;
-    my $idx = -1;
-    while (1)
-    {
-        $offset = $size if $offset > $size;
-        seek _FH, -$offset, 2 or die qq[seek failed on "$progname": $!];
-        my $nread = read _FH, $buf, $offset;
-        die qq[read failed on "$progname": $!] unless $nread == $offset;
-        $idx = rindex($buf, "\nPAR.pm\n");
-        last if $idx >= 0 || $offset == $size || $offset > 256 * 1024;
-        $offset *= 2;
+    my $magic_pos = find_par_magic(*_FH);
+    if ($magic_pos < 0) {
+        outs(qq[Can't find magic string "$PAR_MAGIC" in file "$progname"]);
+        last MAGIC;
     }
-    last unless $idx >= 0;
+    outs("Found PAR magic at position $magic_pos");
 
-    # Seek 4 bytes backward from the signature to get the offset of the 
+    # Check for "\0CACHE" 4 bytes below the signature
+    seek _FH, $magic_pos - $FILE_offset_size - length("\0CACHE"), 0;
+    read _FH, $buf, length("\0CACHE");
+    if ($buf ne "\0CACHE" && $buf ne "\0CLEAN") {
+        outs("No cache marker found");
+        last MAGIC;
+    }
+    outs(qq[Cache marker "$buf" found]);
+
+    # Seek 4 bytes backward from the signature to get the offset of the
     # first embedded FILE, then seek to it
-    $offset -= $idx - 4;
-    seek _FH, -$offset, 2;
-    read _FH, $buf, 4;
-    seek _FH, -$offset - unpack("N", $buf), 2;
-    read _FH, $buf, 4;
+    seek _FH, $magic_pos - $FILE_offset_size, 0;
+    read _FH, $buf, $FILE_offset_size;
+    my $offset = unpack("N", $buf);
+    outs("Offset from start of FILEs is $offset");
+    seek _FH, $magic_pos - $FILE_offset_size - $offset, 0;
+    $start_of_FILE_section = tell _FH;
 
-    $data_pos = (tell _FH) - 4;
     # }}}
 
     # Extracting each file into memory {{{
     my %require_list;
+    read _FH, $buf, 4;                           # read the first "FILE"
     while ($buf eq "FILE") {
         read _FH, $buf, 4;
         read _FH, $buf, unpack("N", $buf);
 
         my $fullname = $buf;
-        outs(qq(Unpacking file "$fullname"...));
+        outs(qq[Unpacking FILE "$fullname"...]);
         my $crc = ( $fullname =~ s|^([a-f\d]{8})/|| ) ? $1 : undef;
         my ($basename, $ext) = ($buf =~ m|(?:.*/)?(.*)(\..*)|);
 
@@ -272,21 +312,17 @@ my ($start_pos, $data_pos);
         read _FH, $buf, unpack("N", $buf);
 
         if (defined($ext) and $ext !~ /\.(?:pm|pl|ix|al)$/i) {
-            my $filename = _tempfile("$crc$ext", $buf, 0755);
+            my $filename = _save_as("$crc$ext", $buf, 0755);
             $PAR::Heavy::FullCache{$fullname} = $filename;
             $PAR::Heavy::FullCache{$filename} = $fullname;
         }
-        elsif ( $fullname =~ m|^/?shlib/| and defined $ENV{PAR_TEMP} ) {
-            my $filename = _tempfile("$basename$ext", $buf, 0755);
-            outs("SHLIB: $filename\n");
-        }
         else {
-            $require_list{$fullname} =
-            $PAR::Heavy::ModuleCache{$fullname} = {
+            $ModuleCache{$fullname} = {
                 buf => $buf,
                 crc => $crc,
                 name => $fullname,
             };
+            $require_list{$fullname}++;
         }
         read _FH, $buf, 4;
     }
@@ -297,26 +333,29 @@ my ($start_pos, $data_pos);
 
         return if ref $module or !$module;
 
-        my $info = delete $require_list{$module} or return;
+        my $info = $ModuleCache{$module};
+        return unless $info;
+
+        delete $require_list{$module};
 
         $INC{$module} = "/loader/$info/$module";
 
         if ($ENV{PAR_CLEAN} and defined(&IO::File::new)) {
-            my $fh = IO::File->new_tmpfile or die $!;
-            binmode($fh);
-            print $fh $info->{buf};
-            seek($fh, 0, 0);
+            my $fh = IO::File->new_tmpfile or die "Can't create temp file: $!";
+            $fh->binmode();
+            $fh->print($info->{buf});
+            $fh->seek(0, 0);
             return $fh;
         }
         else {
-            my $filename = _tempfile("$info->{crc}.pm", $info->{buf});
+            my $filename = _save_as("$info->{crc}.pm", $info->{buf});
+            $info->{file} = $filename;
 
-            open my $fh, '<', $filename or die "can't read $filename: $!";
-            binmode($fh);
+            open my $fh, '<:raw', $filename or die qq[Can't read "$filename": $!];
             return $fh;
         }
 
-        die "Bootstrapping failed: cannot find $module!\n";
+        die "Bootstrapping failed: can't find module $module!";
     }, @INC);
 
     # Now load all bundled files {{{
@@ -349,8 +388,12 @@ my ($start_pos, $data_pos);
 
     # }}}
 
-    last unless $buf eq "PK\003\004";
-    $start_pos = (tell _FH) - 4;
+    if ($buf ne $PKZIP_MAGIC) {
+        outs(qq[No zip found after FILE section in file "$progname"]);
+        last MAGIC ;
+    }
+
+    $start_pos = (tell _FH) - 4;                # start of zip
 }
 # }}}
 
@@ -360,7 +403,6 @@ my ($out, $bundle, $logfh, $cache_name);
 
 delete $ENV{PAR_APP_REUSE}; # sanitize (REUSE may be a security problem)
 
-$quiet = 0 unless $ENV{PAR_DEBUG};
 # Don't swallow arguments for compiled executables without --par-options
 if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
     my %dist_cmd = qw(
@@ -406,7 +448,7 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
                 $quiet = 1;
             }
             elsif ($1 eq 'L') {
-                open $logfh, ">>", $2 or die "XXX: Cannot open log: $!";
+                open $logfh, ">>", $2 or die qq[Can't open log file "$2": $!];
             }
             elsif ($1 eq 'T') {
                 $cache_name = $2;
@@ -446,21 +488,17 @@ if ($out) {
 
 
     if (defined $par) {
-        # increase the chunk size for Archive::Zip so that it will find the EOCD
-        # even if more stuff has been appended to the .par
-        Archive::Zip::setChunkSize(128*1024);
-
-        open my $fh, '<', $par or die "Cannot find '$par': $!";
-        binmode($fh);
+        open my $fh, '<:raw', $par or die qq[Can't find par file "$par": $!];
         bless($fh, 'IO::File');
 
         $zip = Archive::Zip->new;
         ( $zip->readFromFileHandle($fh, $par) == Archive::Zip::AZ_OK() )
-            or die "Read '$par' error: $!";
+            or die qq[Error reading zip archive "$par"];
     }
 
 
-    my %env = do {
+    # Extract the "par" dictionary from META.yml in $zip
+    my %meta_par = do {
         if ($zip and my $meta = $zip->contents('META.yml')) {
             $meta =~ s/.*^par:$//ms;
             $meta =~ s/^\S.*//ms;
@@ -469,41 +507,41 @@ if ($out) {
     };
 
     # Open input and output files {{{
-    local $/ = \4;
 
     if (defined $par) {
-        open PAR, '<', $par or die "$!: $par";
-        binmode(PAR);
-        die "$par is not a PAR file" unless <PAR> eq "PK\003\004";
+        open my $ph, '<:raw', $par or die qq[Can't read par file "$par": $!];
+        my $buf;
+        read $ph, $buf, 4;
+        die qq["$par" is not a par file] unless $buf eq $PKZIP_MAGIC;
+        close $ph;
     }
 
     CreatePath($out) ;
-    
+
     my $fh = IO::File->new(
         $out,
-        IO::File::O_CREAT() | IO::File::O_WRONLY() | IO::File::O_TRUNC(),
+        IO::File::O_CREAT() | IO::File::O_RDWR() | IO::File::O_TRUNC(),
         0777,
-    ) or die $!;
-    binmode($fh);
+    ) or die qq[Can't create file "$out": $!];
+    $fh->binmode();
 
-    $/ = (defined $data_pos) ? \$data_pos : undef;
     seek _FH, 0, 0;
-    my $loader = scalar <_FH>;
+
+    my $loader;
+    if (defined $start_of_FILE_section) {
+        read _FH, $loader, $start_of_FILE_section;
+    } else {
+        local $/ = undef;
+        $loader = <_FH>;
+    }
+
     if (!$ENV{PAR_VERBATIM} and $loader =~ /^(?:#!|\@rem)/) {
         require PAR::Filter::PodStrip;
-        PAR::Filter::PodStrip->new->apply(\$loader, $0)
+        PAR::Filter::PodStrip->apply(\$loader, $0);
     }
-    foreach my $key (sort keys %env) {
-        my $val = $env{$key} or next;
-        $val = eval $val if $val =~ /^['"]/;
-        my $magic = "__ENV_PAR_" . uc($key) . "__";
-        my $set = "PAR_" . uc($key) . "=$val";
-        $loader =~ s{$magic( +)}{
-            $magic . $set . (' ' x (length($1) - length($set)))
-        }eg;
-    }
-    $fh->print($loader);
-    $/ = undef;
+
+    $fh->print($loader)
+        or die qq[Error writing loader to "$out": $!];
     # }}}
 
     # Write bundled modules {{{
@@ -515,100 +553,117 @@ if ($out) {
 
         require_modules();
 
-        my @inc = grep { !/BSDPAN/ } 
+        # NOTE: use $Config::Config{...} since we only "require" (*not* "use") it - no import
+        my @inc = grep { !/BSDPAN/ }
                        grep {
                            ($bundle ne 'site') or
                            ($_ ne $Config::Config{archlibexp} and
                            $_ ne $Config::Config{privlibexp});
                        } @INC;
 
+        # normalize paths (remove trailing or multiple consecutive slashes)
+        s|/+|/|g, s|/$|| foreach @inc;
+
         # Now determine the files loaded above by require_modules():
         # Perl source files are found in values %INC and DLLs are
         # found in @DynaLoader::dl_shared_objects.
         my %files;
         $files{$_}++ for @DynaLoader::dl_shared_objects, values %INC;
 
-        my $lib_ext = $Config::Config{lib_ext};
+        my $lib_ext = $Config::Config{lib_ext};         # XXX lib_ext vs dlext ?
         my %written;
 
-        foreach (sort keys %files) {
-            my ($name, $file);
+        foreach my $key (sort keys %files) {
+            my ($file, $name);
 
-            foreach my $dir (@inc) {
-                if ($name = $PAR::Heavy::FullCache{$_}) {
-                    $file = $_;
-                    last;
-                }
-                elsif (/^(\Q$dir\E\/(.*[^Cc]))\Z/i) {
-                    ($file, $name) = ($1, $2);
-                    last;
-                }
-                elsif (m!^/loader/[^/]+/(.*[^Cc])\Z!) {
-                    if (my $ref = $PAR::Heavy::ModuleCache{$1}) {
-                        ($file, $name) = ($ref, $1);
+            if (defined(my $fc = $PAR::Heavy::FullCache{$key})) {
+                ($file, $name) = ($key, $fc);
+            }
+            else {
+                foreach my $dir (@inc) {
+                    if ($key =~ m|^\Q$dir\E/(.*)$|i) {
+                        ($file, $name) = ($key, $1);
                         last;
                     }
-                    elsif (-f "$dir/$1") {
-                        ($file, $name) = ("$dir/$1", $1);
-                        last;
+                    if ($key =~ m|^/loader/[^/]+/(.*)$|) {
+                        if (my $ref = $ModuleCache{$1}) {
+                            ($file, $name) = ($ref, $1);
+                            last;
+                        }
+                        if (-f "$dir/$1") {
+                            ($file, $name) = ("$dir/$1", $1);
+                            last;
+                        }
                     }
                 }
             }
+            # There are legitimate reasons why we couldn't find $name and $file for a $key:
+            # - cperl has e.g. $INC{"XSLoader.pm"} = "XSLoader.c",
+            #   $INC{"DynaLoader.pm"}' = "dlboot_c.PL"
+            next unless defined $name;
 
-            next unless defined $name and not $written{$name}++;
-            next if !ref($file) and $file =~ /\.\Q$lib_ext\E$/;
-            outs( join "",
-                qq(Packing "), ref $file ? $file->{name} : $file,
-                qq("...)
-            );
+            next if $written{$name}++;
+            next if !ref($file) and $file =~ /\.\Q$lib_ext\E$/i;
 
+            outs(sprintf(qq[Packing FILE "%s"...], ref $file ? $file->{name} : $file));
             my $content;
             if (ref($file)) {
                 $content = $file->{buf};
             }
             else {
-                open FILE, '<', $file or die "Can't open $file: $!";
-                binmode(FILE);
-                $content = <FILE>;
-                close FILE;
+                local $/ = undef;
+                open my $th, '<:raw', $file or die qq[Can't read "$file": $!];
+                $content = <$th>;
+                close $th;
 
-                PAR::Filter::PodStrip->new->apply(\$content, $file)
+                PAR::Filter::PodStrip->apply(\$content, "<embedded>/$name")
                     if !$ENV{PAR_VERBATIM} and $name =~ /\.(?:pm|ix|al)$/i;
 
                 PAR::Filter::PatchContent->new->apply(\$content, $file, $name);
             }
 
-            outs(qq(Written as "$name"));
-            $fh->print("FILE");
-            $fh->print(pack('N', length($name) + 9));
-            $fh->print(sprintf(
-                "%08x/%s", Archive::Zip::computeCRC32($content), $name
-            ));
-            $fh->print(pack('N', length($content)));
-            $fh->print($content);
+            $fh->print("FILE",
+                       pack('N', length($name) + 9),
+                       sprintf("%08x/%s", Archive::Zip::computeCRC32($content), $name),
+                       pack('N', length($content)),
+                       $content)
+                or die qq[Error writing embedded FILE to "$out": $!];
+
+            outs(qq[Written as "$name"]);
         }
     }
     # }}}
 
     # Now write out the PAR and magic strings {{{
-    $zip->writeToFileHandle($fh) if $zip;
-
-    $cache_name = substr $cache_name, 0, 40;
-    if (!$cache_name and my $mtime = (stat($out))[9]) {
-        my $ctx = Digest::SHA->new(1);
-        open(my $fh, "<", $out);
-        binmode($fh);
-        $ctx->addfile($fh);
-        close($fh);
-
-        $cache_name = $ctx->hexdigest;
+    if ($zip) {
+        $zip->writeToFileHandle($fh) == Archive::Zip::AZ_OK
+            or die qq[Error writing zip part of "$out"];
     }
-    $cache_name .= "\0" x (41 - length $cache_name);
-    $cache_name .= "CACHE";
-    $fh->print($cache_name);
-    $fh->print(pack('N', $fh->tell - length($loader)));
-    $fh->print("\nPAR.pm\n");
-    $fh->close;
+
+    if ($meta_par{clean}) {
+        $fh->print("\0CLEAN");
+    }
+    else {
+        if (!defined $cache_name) {
+            my $ctx = Digest::SHA->new(1);
+            seek $fh, 0, 0;
+            $ctx->addfile($fh);
+            seek $fh, 0, 2;
+            $cache_name = $ctx->hexdigest;
+        }
+        # truncate $cache_name to 40 byes and fill with NULs if necessary
+        $cache_name = substr $cache_name, 0, $cache_name_size;
+        $cache_name .= "\0" x ($cache_name_size - length $cache_name);
+        $fh->print($cache_name, 
+                   "\0CACHE");
+    }
+
+    # compute the offset from the end of $loader to end of "...\0CACHE" or "\0CLEAN"
+    my $offset = $fh->tell - length($loader);
+    $fh->print(pack('N', $offset),
+               $PAR_MAGIC);
+
+    $fh->close or die qq[Error writing trailer of "$out": $!];
     chmod 0755, $out;
     # }}}
 
@@ -633,23 +688,22 @@ if ($out) {
         require Archive::Zip;
     }
 
-    # increase the chunk size for Archive::Zip so that it will find the EOCD
-    # even if more stuff (OS-specific codesigning, for example) has been appended to the pp exe
-    # OSX codesign tool appends at least 180K to a binary and so make the ChunkSize generously
-    # greater than this
-    Archive::Zip::setChunkSize(256*1024);
+    my $fh = IO::File->new;                             # Archive::Zip operates on an IO::Handle
+    $fh->fdopen(fileno(_FH), 'r') or die qq[fdopen() failed: $!];
 
+    # Temporarily increase the chunk size for Archive::Zip so that it will find the EOCD
+    # even if lots of stuff has been appended to the pp'ed exe (e.g. by OSX codesign).
+    Archive::Zip::setChunkSize(-s _FH);
     my $zip = Archive::Zip->new;
-    my $fh = IO::File->new;
-    $fh->fdopen(fileno(_FH), 'r') or die "$!: $@";
-    $zip->readFromFileHandle($fh, $progname) == Archive::Zip::AZ_OK() or die "$!: $@";
+    ($zip->readFromFileHandle($fh, $progname) == Archive::Zip::AZ_OK())
+        or die qq[Error reading zip archive "$progname"];
+    Archive::Zip::setChunkSize(64 * 1024);
 
     push @PAR::LibCache, $zip;
     $PAR::LibCache{$progname} = $zip;
 
-    $quiet = !$ENV{PAR_DEBUG};
-    outs(qq(\$ENV{PAR_TEMP} = "$ENV{PAR_TEMP}"));
 
+    outs("Extracting zip...");
     if (defined $ENV{PAR_TEMP}) { # should be set at this point!
         foreach my $member ( $zip->members ) {
             next if $member->isDirectory;
@@ -657,7 +711,6 @@ if ($out) {
             next unless $member_name =~ m{
                 ^
                 /?shlib/
-                (?:$Config::Config{version}/)?
                 (?:$Config::Config{archname}/)?
                 ([^/]+)
                 $
@@ -665,10 +718,11 @@ if ($out) {
             my $extract_name = $1;
             my $dest_name = File::Spec->catfile($ENV{PAR_TEMP}, $extract_name);
             if (-f $dest_name && -s _ == $member->uncompressedSize()) {
-                outs(qq(Skipping "$member_name" since it already exists at "$dest_name"));
+                outs(qq[Skipping "$member_name" since it already exists at "$dest_name"]);
             } else {
-                outs(qq(Extracting "$member_name" to "$dest_name"));
-                $member->extractToFileNamed($dest_name);
+                outs(qq[Extracting "$member_name" to "$dest_name"...]);
+                ($member->extractToFileNamed($dest_name) == Archive::Zip::AZ_OK())
+                    or die qq[Error extracting zip member to "$dest_name"];
                 chmod(0555, $dest_name) if $^O eq "hpux";
             }
         }
@@ -689,17 +743,16 @@ Usage: $0 [ -Alib.par ] [ -Idir ] [ -Mmodule ] [ src.par ] [ program.pl ]
 
 sub CreatePath {
     my ($name) = @_;
-    
+
     require File::Basename;
     my ($basename, $path, $ext) = File::Basename::fileparse($name, ('\..*'));
-    
+
     require File::Path;
-    
+
     File::Path::mkpath($path) unless(-e $path); # mkpath dies with error
 }
 
 sub require_modules {
-    #local $INC{'Cwd.pm'} = __FILE__ if $^O ne 'MSWin32';
 
     require lib;
     require DynaLoader;
@@ -764,39 +817,42 @@ sub _set_par_temp {
         else {
             $username = $ENV{USERNAME} || $ENV{USER} || 'SYSTEM';
         }
-        $username =~ s/\W/_/g;
 
-        my $stmpdir = "$path$Config{_delim}par-".unpack("H*", $username);
+        my $stmpdir = "$path$sys{_delim}par-".unpack("H*", $username);
         mkdir $stmpdir, 0755;
-        if (!$ENV{PAR_CLEAN} and my $mtime = (stat($progname))[9]) {
-            open (my $fh, "<". $progname);
-            seek $fh, -18, 2;
-            sysread $fh, my $buf, 6;
-            if ($buf eq "\0CACHE") {
-                seek $fh, -58, 2;
-                sysread $fh, $buf, 41;
-                $buf =~ s/\0//g;
-                $stmpdir .= "$Config{_delim}cache-" . $buf;
-            }
-            else {
-                require Digest::SHA; 
-                my $ctx = Digest::SHA->new(1);
-                open(my $fh, "<", $progname);
-                binmode($fh);
-                $ctx->addfile($fh);
-                close($fh);
 
-                $stmpdir .= "$Config{_delim}cache-" . $ctx->hexdigest;
-            }
-            close($fh);
+        my $cache_dir;
+        if ($ENV{PAR_CLEAN}) {
+            $cache_dir = "temp-$$";
         }
         else {
-            $ENV{PAR_CLEAN} = 1;
-            $stmpdir .= "$Config{_delim}temp-$$";
+            open my $fh, "<:raw", $progname or die qq[Can't read "$progname": $!];
+            if ((my $magic_pos = find_par_magic($fh)) >= 0) {
+                seek $fh, $magic_pos 
+                          - $FILE_offset_size 
+                          - length("\0CACHE"), 0;
+                my $buf;
+                read $fh, $buf, length("\0CACHE");
+                if ($buf eq "\0CACHE") {
+                    seek $fh, $magic_pos 
+                              - $FILE_offset_size 
+                              - length("\0CACHE") 
+                              - $cache_name_size, 0;
+                    read $fh, $buf, $cache_name_size;
+                    $buf =~ s/\0//g;
+                    $cache_dir = "cache-$buf";
+                }
+            }
+            close $fh;
         }
+        if (!$cache_dir) {
+            $cache_dir = "temp-$$";
+            $ENV{PAR_CLEAN} = 1;
+        }
+        $stmpdir .= "$sys{_delim}$cache_dir";
 
-        $ENV{PAR_TEMP} = $stmpdir;
         mkdir $stmpdir, 0755;
+        $ENV{PAR_TEMP} = $stmpdir;
         last;
     }
 
@@ -805,27 +861,28 @@ sub _set_par_temp {
 
 
 # check if $name (relative to $par_temp) already exists;
-# if not, create a file with a unique temporary name, 
+# if not, create a file with a unique temporary name,
 # fill it with $contents, set its file mode to $mode if present;
-# finaly rename it to $name; 
+# finaly rename it to $name;
 # in any case return the absolute filename
-sub _tempfile {
+sub _save_as {
     my ($name, $contents, $mode) = @_;
 
     my $fullname = "$par_temp/$name";
     unless (-e $fullname) {
         my $tempname = "$fullname.$$";
 
-        open my $fh, '>', $tempname or die "can't write $tempname: $!";
-        binmode $fh;
-        print $fh $contents;
-        close $fh;
+        my $fh;
+        (open $fh, '>:raw', $tempname)
+        && (print $fh $contents)
+        && (close $fh)
+            or die qq[Error writing "$tempname": $!];
         chmod $mode, $tempname if defined $mode;
 
         rename($tempname, $fullname) or unlink($tempname);
-        # NOTE: The rename() error presumably is something like ETXTBSY 
+        # NOTE: The rename() error presumably is something like ETXTBSY
         # (scenario: another process was faster at extraction $fullname
-        # than us and is already using it in some way); anyway, 
+        # than us and is already using it in some way); anyway,
         # let's assume $fullname is "good" and clean up our copy.
     }
 
@@ -841,40 +898,40 @@ sub _set_progname {
     $progname ||= $0;
 
     if ($ENV{PAR_TEMP} and index($progname, $ENV{PAR_TEMP}) >= 0) {
-        $progname = substr($progname, rindex($progname, $Config{_delim}) + 1);
+        $progname = substr($progname, rindex($progname, $sys{_delim}) + 1);
     }
 
-    if (!$ENV{PAR_PROGNAME} or index($progname, $Config{_delim}) >= 0) {
+    if (!$ENV{PAR_PROGNAME} or index($progname, $sys{_delim}) >= 0) {
         if (open my $fh, '<', $progname) {
             return if -s $fh;
         }
-        if (-s "$progname$Config{_exe}") {
-            $progname .= $Config{_exe};
+        if (-s "$progname$sys{_exe}") {
+            $progname .= $sys{_exe};
             return;
         }
     }
 
-    foreach my $dir (split /\Q$Config{path_sep}\E/, $ENV{PATH}) {
+    foreach my $dir (split /\Q$sys{path_sep}\E/, $ENV{PATH}) {
         next if exists $ENV{PAR_TEMP} and $dir eq $ENV{PAR_TEMP};
-        $dir =~ s/\Q$Config{_delim}\E$//;
-        (($progname = "$dir$Config{_delim}$progname$Config{_exe}"), last)
-            if -s "$dir$Config{_delim}$progname$Config{_exe}";
-        (($progname = "$dir$Config{_delim}$progname"), last)
-            if -s "$dir$Config{_delim}$progname";
+        $dir =~ s/\Q$sys{_delim}\E$//;
+        (($progname = "$dir$sys{_delim}$progname$sys{_exe}"), last)
+            if -s "$dir$sys{_delim}$progname$sys{_exe}";
+        (($progname = "$dir$sys{_delim}$progname"), last)
+            if -s "$dir$sys{_delim}$progname";
     }
 }
 
 sub _fix_progname {
     $0 = $progname ||= $ENV{PAR_PROGNAME};
-    if (index($progname, $Config{_delim}) < 0) {
-        $progname = ".$Config{_delim}$progname";
+    if (index($progname, $sys{_delim}) < 0) {
+        $progname = ".$sys{_delim}$progname";
     }
 
     # XXX - hack to make PWD work
     my $pwd = (defined &Cwd::getcwd) ? Cwd::getcwd()
                 : ((defined &Win32::GetCwd) ? Win32::GetCwd() : `pwd`);
     chomp($pwd);
-    $progname =~ s/^(?=\.\.?\Q$Config{_delim}\E)/$pwd$Config{_delim}/;
+    $progname =~ s/^(?=\.\.?\Q$sys{_delim}\E)/$pwd$sys{_delim}/;
 
     $ENV{PAR_PROGNAME} = $progname;
 }
@@ -887,21 +944,16 @@ sub _par_init_env {
     }
 
     for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ) ) {
-        delete $ENV{'PAR_'.$_};
-    }
-    for (qw/ TMPDIR TEMP CLEAN DEBUG /) {
-        $ENV{'PAR_'.$_} = $ENV{'PAR_GLOBAL_'.$_} if exists $ENV{'PAR_GLOBAL_'.$_};
+        delete $ENV{"PAR_$_"};
     }
 
-    my $par_clean = "__ENV_PAR_CLEAN__               ";
+    for (qw/ TMPDIR TEMP CLEAN DEBUG /) {
+        $ENV{"PAR_$_"} = $ENV{"PAR_GLOBAL_$_"} if exists $ENV{"PAR_GLOBAL_$_"};
+    }
 
     if ($ENV{PAR_TEMP}) {
         delete $ENV{PAR_CLEAN};
     }
-    elsif (!exists $ENV{PAR_GLOBAL_CLEAN}) {
-        my $value = substr($par_clean, 12 + length("CLEAN"));
-        $ENV{PAR_CLEAN} = $1 if $value =~ /^PAR_CLEAN=(\S+)/;
-    }
 }
 
 sub outs {
@@ -916,6 +968,7 @@ sub outs {
 
 sub init_inc {
     require Config;
+    # NOTE: use $Config::Config{...} since we only "require" (*not* "use") it - no import
     push @INC, grep defined, map $Config::Config{$_}, qw(
         archlibexp privlibexp sitearchexp sitelibexp
         vendorarchexp vendorlibexp
@@ -931,7 +984,7 @@ require PAR;
 unshift @INC, \&PAR::find_par;
 PAR->import(@par_args);
 
-die qq(par.pl: Can't open perl script "$progname": No such file or directory\n)
+die qq[par.pl: Can't open perl script "$progname": No such file or directory\n]
     unless -e $progname;
 
 do $progname;
@@ -988,4 +1041,4 @@ See F<LICENSE>.
 
 =cut
 
-__END__
+ __END__
